#include "profiles.h"

#include <QStandardItemModel>
#include <QPushButton>


Profiles::Profiles(QWidget* parent)
    :QDialog(parent)
{
    setupUi(this);

    buttonBox->button(QDialogButtonBox::Ok)->setText(tr("&OK"));
    buttonBox->button(QDialogButtonBox::Cancel)->setText(tr("&Cancel"));

    mProfModel = new QStandardItemModel(this);

    profileList->setModel(mProfModel);

    connect(profileList->selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)),
            this, SLOT(currentChanged(QModelIndex, QModelIndex)));
}

void Profiles::setProfiles(const QList<Profile> &profs)
{
    int idx = 0;

    foreach (const Profile& p, profs)
    {
        QStandardItem* i;
        QList<QStandardItem*> item;

        i = new QStandardItem(p.Title);
        item.append(i);

        i = new QStandardItem(p.Server);
        item.append(i);

        i = new QStandardItem();
        i->setData(p.Dir, Qt::UserRole);
        item.append(i);

        i = new QStandardItem();
        i->setData(idx++, Qt::UserRole);
        item.append(i);

        mProfModel->appendRow(item);
    }

    profileList->header()->hideSection(2);
    profileList->header()->hideSection(3);

    mProfModel->setHeaderData(0, Qt::Horizontal, tr("Profile name"), Qt::DisplayRole);
    mProfModel->setHeaderData(1, Qt::Horizontal, tr("Vim name"), Qt::DisplayRole);
}

QList<Profile> Profiles::getProfiles()
{
    QList<Profile> prof;

    for (int i=0, c=mProfModel->rowCount(); i<c; i++)
    {
        prof.append(Profile(
                        mProfModel->index(i, 0).data(Qt::DisplayRole).toString(),
                        mProfModel->index(i, 1).data(Qt::DisplayRole).toString(),
                        mProfModel->index(i, 2).data(Qt::UserRole).toString()));
    }

    return prof;
}

int Profiles::getNewIndex(int index)
{
    for (int i=0, c=mProfModel->rowCount(); i<c; i++)
    {
        const QVariant idx = mProfModel->index(i, 3).data(Qt::UserRole);

        if (idx.isValid() && idx.toInt() == index)
        {
            return i;
        }
    }

    return -1;
}

/* --------------------------------------------------------------------- */

void Profiles::on_addProfile_clicked()
{
    QStandardItem* i;
    QList<QStandardItem*> item;

    i = new QStandardItem(tr("New profile"));
    item.append(i);

    i = new QStandardItem("GVIM");
    item.append(i);

    i = new QStandardItem();
    item.append(i);

    i = new QStandardItem();
    item.append(i);

    mProfModel->appendRow(item);

    profileList->selectionModel()->setCurrentIndex(mProfModel->index(mProfModel->rowCount() - 1, 0),
        QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);

    profileList->edit(mProfModel->index(mProfModel->rowCount() - 1, 0));
}

void Profiles::on_removeProfile_clicked()
{
    if (profileList->currentIndex().isValid() && mProfModel->rowCount() > 1)
    {
        mProfModel->removeRow(profileList->currentIndex().row());

        currentChanged(profileList->currentIndex());
    }
}

void Profiles::on_moveUp_clicked()
{
    if (profileList->currentIndex().isValid())
    {
        const int row = profileList->currentIndex().row();

        if (row > 0)
        {
            mProfModel->insertRow(row - 1, mProfModel->takeRow(row));

            profileList->selectionModel()->setCurrentIndex(mProfModel->index(row - 1, 0),
                QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);
        }
    }
}

void Profiles::on_moveDown_clicked()
{
    if (profileList->currentIndex().isValid())
    {
        const int row = profileList->currentIndex().row();

        if (row < mProfModel->rowCount() - 1)
        {
            mProfModel->insertRow(row + 1, mProfModel->takeRow(row));

            profileList->selectionModel()->setCurrentIndex(mProfModel->index(row + 1, 0),
                QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);
        }
    }
}

void Profiles::currentChanged(const QModelIndex &index, const QModelIndex &previous)
{
    Q_UNUSED(previous);

    if (index.isValid())
    {
        removeProfile->setEnabled(mProfModel->rowCount() > 1);
        moveUp->setEnabled(index.row() > 0);
        moveDown->setEnabled(index.row() < mProfModel->rowCount() - 1);
    }
    else
    {
        removeProfile->setEnabled(false);
        moveUp->setEnabled(false);
        moveDown->setEnabled(false);
    }
}
