#ifndef VIMPAL_H
#define VIMPAL_H

#include <QMainWindow>
#include <QStack>

#include "profile.h"
#include "tool.h"

class QFileSystemModel;
class QModelIndex;
class QTreeView;
class QComboBox;
class QMenu;

class VimPal: public QMainWindow
{
    Q_OBJECT

public:
    VimPal(QWidget* parent = 0, Qt::WindowFlags f = 0);

protected:
    void closeEvent(QCloseEvent* e);

private slots:
    void goHome();
    void goUp();
    void goBack();
    void goForward();
    void changeRoot(QAction* action);

    void updateRootMenu();

    void fileClick(const QModelIndex& idx);
    void fileDblClick(const QModelIndex& idx);
    void contextMenu(const QPoint& pos);

    void editPreferences();
    void editProfiles();

    void profileChanged(int prof);

    void showAbout();

private:
    bool vimCmd(const QString& cmd, bool startNew);
    bool vimServerExists(const QString& name);
    void selectFile(const QModelIndex& idx);
    void selectDir(const QModelIndex& idx, bool noHistory = false);
    void selectDir(const QString& dir, bool noHistory = false);
    void deleteFile(const QModelIndex& idx);
    void renameFile(const QModelIndex& idx);
    void newFile(const QModelIndex& idx);
    void newDirectory(const QModelIndex& idx);
    void updateProfiles();
    void updateColors();

    void clearHistory();
    void pushHistory(const QString& path);

    QString getDir(const QModelIndex& idx);

    QStringList getExpanded(const QModelIndex& parent);

private:
    QFileSystemModel* mDirModel;
    QTreeView* mDirView;
    QComboBox* mProfileBox;
    QMenu* mRootMenu;
    QAction* mGoBack,
           * mGoForward;

    QToolBar* mNavigationToolbar,
            * mProfileToolbar;

    bool mSingleClickOpen,
         mChangeDir,
         mOpenNewTabs,
         mDarkBackground,
         mShowHidden;

    QString mVimCommand;

    int mCurrentProfile,
        mLastPressedButton;

    QList<Profile> mProfiles;
    QList<Tool> mTools;
    QStack<QString> mHistory, mForwardHistory;
};

#endif   // VIMPAL_H
